/*
 *   feactal.h
 *   NoQuaFra - Non Quaternion 3d Fractal Generator
 *   Copyright (C) 2002 
 *
 This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *
 *   Billy Stiltner
 *   billystiltner@sourceforge.net
 */

//fractal.h
//written by Billy Stiltner
//started out as 2d mandelbrot class 
//the earliest last modified date of the 3d fractal class was 2-22-95 
//I found files that lead me to believe 
//I started it around 4-94



struct myvertex{
public:
float x,y,z;
unsigned char r,g,b,a;
unsigned char n;
};

#define   drawvertexline(v1,v2) glBegin(GL_LINES); \
 glVertex3f ((v1.x),(v1.y),(v1.z)); glVertex3f ((v2.x),(v2.y),(v2.z)); glEnd();


class fractal{
 public:
 double x1,y1,z1;
 double x2,y2,z2;
 double realp,imagp,kapap;
 int startx,starty,startz;
 int width,height,depth;

 double magnitude;
 int numits;

int nvertexes;

 float scaledivisor;
 float scx,scy,scz;
int lowcolor,highcolor;

int alphamode;
int julia;
myvertex vtx[32768000];
myvertex cube[9];
myvertex zoomcube[9];

myvertex newmin;
myvertex newmax;

myvertex u,v,w;



 void init(double x_1, double y_1, double z_1,
           double x_2, double y_2, double z_2,
           double rp, double ip, double kp,
           int sstartx, int sstarty, int sstartz,
           int ww, int hh, int dd,           
           double mmagnitude,
           int nnumits);



void setcube(void);
void drawcube(void);
void setzoomcube(void);
void drawzoomcube(void);
void scalezoomcube(float tx,float ty,float tz);
void translatezoomcube(float tx,float ty,float tz);
void zoom(void);


};




class mandelbrot:public fractal{
public:



 void iterate(void);

};


void fractal::init(double x_1, double y_1, double z_1,
                 double x_2, double y_2, double z_2,
                 double rp, double ip, double kp,
                 int sstartx, int sstarty, int sstartz,
                 int ww, int hh, int dd,
                 double mmagnitude,
                 int nnumits)
 {
 x1=x_1; y1=y_1; z1=z_1;
 x2=x_2; y2=y_2; z2=z_2;

 realp=rp;
 imagp=ip;
 kapap=kp; 

 startx=sstartx;
 starty=sstarty;
 startz=sstartz; 

 width=ww;
 height=hh;
 depth=dd;

 magnitude=mmagnitude;
 numits=nnumits;

 lowcolor=-1;
 highcolor=255;

 alphamode=0;
 julia=0;

 //set this value so our 3d model can be scaled 
 scaledivisor=1;
 u.x=0;
 u.y=0;
 u.z=0;

 v.x=width;
 v.y=height;
 v.z=depth;


w.x=(v.x-u.x)/2;
w.y=(v.y-u.y)/2;
w.z=(v.z-u.z)/2;

scx=width;
scy=height;
scz=depth;

 }

void fractal::setcube(void)
{
 cube[1].x=0.0;
 cube[1].y=scy;
 cube[1].z=0.0; 

 cube[2].x=scx;
 cube[2].y=scy;
 cube[2].z=0.0;

 cube[3].x=scx;
 cube[3].y=0.0;
 cube[3].z=0.0;

 cube[4].x=0.0;
 cube[4].y=0.0;
 cube[4].z=0.0;

 cube[5].x=0.0;
 cube[5].y=scy;
 cube[5].z=scz;

 cube[6].x=scx;
 cube[6].y=scy;
 cube[6].z=scz;

 cube[7].x=scx;
 cube[7].y=0.0;
 cube[7].z=scz;

 cube[8].x=0.0;
 cube[8].y=0.0;
 cube[8].z=scz;

 
}

void fractal::drawcube(void)
{
glColor4ub (255,255,255,255);


 drawvertexline(cube[1],cube[2]);
 drawvertexline(cube[2],cube[3]);
 drawvertexline(cube[3],cube[4]);
 drawvertexline(cube[1],cube[4]);
 drawvertexline(cube[5],cube[6]);
 drawvertexline(cube[6],cube[7]);
 drawvertexline(cube[7],cube[8]);
 drawvertexline(cube[5],cube[8]);
 drawvertexline(cube[1],cube[5]);
 drawvertexline(cube[2],cube[6]);
 drawvertexline(cube[3],cube[7]);
 drawvertexline(cube[4],cube[8]);
}


void fractal::setzoomcube(void)
{
 zoomcube[1].x=0.0;
 zoomcube[1].y=scy;
 zoomcube[1].z=0.0; 

 zoomcube[2].x=scx;
 zoomcube[2].y=scy;
 zoomcube[2].z=0.0;

 zoomcube[3].x=scx;
 zoomcube[3].y=0.0;
 zoomcube[3].z=0.0;

 zoomcube[4].x=0.0;
 zoomcube[4].y=0.0;
 zoomcube[4].z=0.0;

 zoomcube[5].x=0.0;
 zoomcube[5].y=scy;
 zoomcube[5].z=scz;

 zoomcube[6].x=scx;
 zoomcube[6].y=scy;
 zoomcube[6].z=scz;

 zoomcube[7].x=scx;
 zoomcube[7].y=0.0;
 zoomcube[7].z=scz;

 zoomcube[8].x=0.0;
 zoomcube[8].y=0.0;
 zoomcube[8].z=scz;

}

void fractal::drawzoomcube(void)
{
glColor4ub (255,255,0,255);


 drawvertexline(zoomcube[1],zoomcube[2]);
 drawvertexline(zoomcube[2],zoomcube[3]);
 drawvertexline(zoomcube[3],zoomcube[4]);
 drawvertexline(zoomcube[1],zoomcube[4]);
 drawvertexline(zoomcube[5],zoomcube[6]);
 drawvertexline(zoomcube[6],zoomcube[7]);
 drawvertexline(zoomcube[7],zoomcube[8]);
 drawvertexline(zoomcube[5],zoomcube[8]);
 drawvertexline(zoomcube[1],zoomcube[5]);
 drawvertexline(zoomcube[2],zoomcube[6]);
 drawvertexline(zoomcube[3],zoomcube[7]);
 drawvertexline(zoomcube[4],zoomcube[8]);

}


void fractal::translatezoomcube(float tx,float ty,float tz)
{

for(int tn=1;tn<9;tn++)
{
zoomcube[tn].x+=tx;
zoomcube[tn].y+=ty;
zoomcube[tn].z+=tz;
}


 u.x+=tx;
 u.y+=ty;
 u.z+=tz;

 v.x+=tx;
 v.y+=ty;
 v.z+=tz;


w.x+=tx;
w.y+=ty;
w.z+=tz;



}


void fractal::scalezoomcube(float tx,float ty,float tz)
{

myvertex t;

t.x=((v.x-u.x)*tx)/2;
t.y=((v.y-u.y)*ty)/2;
t.z=((v.z-u.z)*tz)/2;

 u.x=w.x-t.x;
 u.y=w.y-t.y;
 u.z=w.z-t.z;

 v.x=w.x+t.x;
 v.y=w.y+t.y;
 v.z=w.z+t.z;


/*
for(int tn=1;tn<9;tn++)
{
zoomcube[tn].x*=tx;
zoomcube[tn].y*=ty;
zoomcube[tn].z*=tz;
}
*/
 zoomcube[1].x=u.x;
 zoomcube[1].y=v.y;
 zoomcube[1].z=u.z; 

 zoomcube[2].x=v.x;
 zoomcube[2].y=v.y;
 zoomcube[2].z=u.z;

 zoomcube[3].x=v.x;
 zoomcube[3].y=u.y;
 zoomcube[3].z=u.z;

 zoomcube[4].x=u.x;
 zoomcube[4].y=u.y;
 zoomcube[4].z=u.z;

 zoomcube[5].x=u.x;
 zoomcube[5].y=v.y;
 zoomcube[5].z=v.z;

 zoomcube[6].x=v.x;
 zoomcube[6].y=v.y;
 zoomcube[6].z=v.z;

 zoomcube[7].x=v.x;
 zoomcube[7].y=u.y;
 zoomcube[7].z=v.z;

 zoomcube[8].x=u.x;
 zoomcube[8].y=u.y;
 zoomcube[8].z=v.z;


}

void fractal::zoom(void)
{
/*
for(int tn=1;tn<9;tn++)
{
cube[tn].x=zoomcube[tn].x;
cube[tn].y=zoomcube[tn].y;
cube[tn].z=zoomcube[tn].z;
}
*/

x1=x1+((x2-x1)*(zoomcube[4].x-startx)/width);
x2=x1+((x2-x1)*(zoomcube[6].x-startx)/width);

y1=y1+((y2-y1)*(zoomcube[4].y-starty)/height);
y2=y1+((y2-y1)*(zoomcube[6].y-starty)/height);

z1=z1+((z2-z1)*(zoomcube[4].z-startz)/height);
z2=z1+((z2-z1)*(zoomcube[6].z-startz)/height);



 u.x=0;
 u.y=0;
 u.z=0;

 v.x=width;
 v.y=height;
 v.z=depth;


w.x=(v.x-u.x)/2;
w.y=(v.y-u.y)/2;
w.z=(v.z-u.z)/2;




}


/////////////////////////////////////////////////////////////////////
void mandelbrot::iterate(void)
{

 int vtxi;

 long int p,q,r;
 int n;
 double k,l,x,y,z,newx,newy,newz;
 double m;

 vtxi=0;

 for(r=1; r<depth; r++)
    {
     for (p=1; p<width; p++)
         {
          for (q=1; q<height; q++)
               {
               if(julia==0){
                k=(double)x1+(x2-x1)*p /(width);
                l=(double)y1+(y2-y1)*q /(height);
                m=(double)z1+(z2-z1)*r /(depth); 
      
                x=realp;
                y=imagp;
                z=kapap;
                }else{

                x=(double)x1+(x2-x1)*p /(width);
                y=(double)y1+(y2-y1)*q /(height);
                z=(double)z1+(z2-z1)*r /(depth); 
      
                k=realp;
                l=imagp;
                m=kapap;
                }

                for (n=1; n<numits; n++)
                    {
                     //newx=x*x-y*y+k;
                     //newy=2*x*y+l;




/*
6-21- 02 equation
XX = (XX1 * XX2) - (YY1 * YY2) - (ZZ1 * ZZ2)
   YY = (YY1 * XX2) + (XX1 * YY2)
   ZZ = (ZZ1 * XX2) + (XX1 * ZZ2)
*/

   newx = ((x * x) - (y * y) - (z *z)) +k;
   newy = ((y *x) + (x *y)) +l;
   newz = ((z * x) + (x * z)) +m;




		     //6-9-2002 equation
           /*         
                     newx=((y*z)+(z*y))+k;
                     newy=((x*z)+(z*x))+l;
                     newz=((x*y)+(y*x))+m;
*/

                      /*
                       *  2000 equation  
                       */
 	
                       /*
		        newx=((x*x)-(y*y)-(z*z))+k;
                        newy=((y*x)-(z*y)+(x*z))+l;
                        newz=((z*x)+(x*y)-(y*z))+m;
                        */

                      /*
                       *  1996 equation - see history 
                       */

                       /*
		       newx=((x*x)-(y*y)-(z*z))+k;
                       newy=((x*y)+(x*y))+l;
                       newz=(x*z)+(x*z)+m;
                     */

	             x=newx;
                     y=newy;
                     z=newz; 

                     //putpixel(surface, p+startx-1,q+starty-1,n);
                     //putpixel(surface,(int)(x*10+160),(int)(y*10+100),(unsigned char)n);

	             if (x*x+y*y+z*z>magnitude)
	                {
                         if((n>lowcolor)&&(n<highcolor))
                           {             
                            vtx[vtxi].x=(p/scaledivisor);
                            vtx[vtxi].y=(q/scaledivisor);
                            vtx[vtxi].z=(r/scaledivisor);
                            vtx[vtxi].r=cMap[n].r;
                            vtx[vtxi].g=cMap[n].g;
                            vtx[vtxi].b=cMap[n].b;
                            
                            if(alphamode==0)
                               {
                                 vtx[vtxi].a=16;
                               }else{ 
                                      vtx[vtxi].a=(unsigned char)128;//((0.9375*n)+16);//((256/(256-16)) * n)+16
                                     }
                            vtx[vtxi].n=n;
                            vtxi++;
                            }



	                  // putpixel(surface, p+startx-1,q+starty-1,n);
                          //SDL_UpdateRect(screen, p+startx-1, q+starty-1, 1, 1);

	                  n=numits;
               
	                 }//mag check

                    }//n

               }//q
         }//p
    }//r
 nvertexes=vtxi;
}
/////////////////////////////////////////////////////////////////////

/*
a: [x1,y1,z1]
b: [x2,y2,z2]

a*b=(y1 * z2 - z1 * y2)i + (x1 * z2 - z1 * x2)j + (x1 * y2 - y1 * x2)k

*/




