/*
	NoQuaFra.cpp
	NoQuaFra - Non Quaternion 3d Fractal Generator
	Copyright (C)Billy Stiltner 2002

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


	Billy Stiltner
	billystiltner@sourceforge.net
*/

//////////////////////////////////////////////////////////////////////
//#include "windows.h"




#include <stdio.h>
#include <stdlib.h>

#include <GL/glx.h>    /* this includes the necessary X headers */
#include <GL/gl.h>

#include <X11/X.h>    /* X11 constant (e.g. TrueColor) */
#include <X11/keysym.h>

#include "noquafraaux.h"

#include "defaultmap.h"
#include "fractal.h"

//////////////////////////////////////////////////////////////////////

#define FRC_POINTS  0
#define FRC_CUBES   1
#define FRC_SPHERES 2
//////////////////////////////////////////////////////////////////////

GLboolean should_rotate;
float angle ;

float anglex ;
float angley ;
float anglez ;
mandelbrot mdel;

int colorcycle;
GLboolean should_colorcycle;
GLboolean rotatex;
GLboolean rotatey;
GLboolean rotatez;
float move_x,xpos;
float move_y,ypos;
float move_z,zpos;
int mouse_left,mouse_right;

int nsize;
int pointstyle;
int pointsize;
GLboolean uselighting;
GLboolean zooming;


//////////////////////////////////////////////////////////////////////

void quit_program( int code );
void handle_key_down( SDL_keysym* keysym );

void process_events( void );

void draw_screen( void );
void setup_opengl( int width, int height );
void MouseMotionHandler(Uint8 type, SDL_MouseMotionEvent event);
void MouseButtonHandler(Uint8 type, SDL_MouseButtonEvent event);

//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
int main( int argc, char* argv[] )
{

	const SDL_VideoInfo* info = NULL;
	int width = 0;
	int height = 0;
	int bpp = 0;
	int flags = 0;

	pointstyle=FRC_POINTS;
	pointsize=10;
	nsize=75;

	should_rotate = GL_FALSE;

	uselighting=GL_FALSE;

	zooming = GL_FALSE;

	angle = 0.0f;
	anglex=0.0f;
	angley=0.0f;
	anglez=0.0f;

	colorcycle=0;
	move_x=0;
	move_y=0;
	move_z=0;

	//this moves the fractal so its center is 0,0,-50
	xpos=-(nsize/2);			 //set this to  - fractal cube width / 2   or move the fractal left half its width
	ypos=-(nsize/2);			 //set this to  - fractal cube height / 2  or move the fractal down half its height
	zpos=-(nsize*2);			 ////set this to  - fractal cube depth * 2 or move the fractal forward twice  its length

	if( SDL_Init( SDL_INIT_VIDEO ) < 0 )
	{
		fprintf( stderr, "Video initialization failed: %s\n",
			SDL_GetError( ) );
		quit_program( 1 );
	}

	info = SDL_GetVideoInfo( );

	if( !info )
	{
		fprintf( stderr, "Video query failed: %s\n",
			SDL_GetError( ) );
		quit_program( 1 );
	}

	width = 640;
	height = 480;
	bpp = info->vfmt->BitsPerPixel;

	SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 8 );
	SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 8 );
	SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 8 );
	SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );

	flags = SDL_OPENGL|SDL_RESIZABLE;
	if(SDL_SetVideoMode( width, height, bpp, flags ) == 0 )
	{
		fprintf( stderr, "Video mode set failed: %s\n",
			SDL_GetError( ) );
		quit_program( 1 );
	}

	setup_opengl( width, height );

	setallcolors();
	//mdel.init(-2.5, 1.5, 2.5,-1.5, 0.0, 0.0, 4, 10000, 0, 0, 100,75);
	//mdel.init(-2.5, 1.5, 2.5,-1.5, 0.0, 0.0, 4, 1000, 0, 0, 100,75);

	//mdel.init(-1.1875, -1.1875, 1.25,1.25, 0.0, 0.0, 4, 1000, 0, 0, 100,100);

	// mdel.init(-1.75, -1.75, 0.0,0.0, 0.0, 0.0, 4, 1000, 0, 0, nsize,nsize);
	mdel.init(-1.75, -1.75, -1.75,
		1.75,  1.75,  1.75,
		0.0,   0.0,   0.0,
		0,     0,     0,
		nsize, nsize, nsize,
		4,
		1000);

	mdel.iterate();
	mdel.setcube();
	mdel.setzoomcube();

	while( 1 )
	{
		process_events( );
		draw_screen( );
	}

	return 0;
}


//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
void draw_screen( void )
{

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity( );

	xpos+=move_x;
	ypos+=move_y;
	zpos+=move_z;

	glTranslatef( xpos, ypos, zpos );
	glRotatef( angle,  1.0, 1.0, 1.0 );

	//glScalef(0.1, 0.1, 0.1);

	if( should_rotate )
	{
		if( ++angle > 360.0f )
		{
			angle = 0.0f;
		}
	}

	glPointSize(pointsize);

	glBegin(GL_LINES);
	mdel.drawcube();
	glEnd();

	glBegin(GL_POINTS);
	for(int v=0;v<mdel.nvertexes;v++)
	{
		int c;
		c=mdel.vtx[v].n;
		c=c+colorcycle;
		c%=255;
		//glColor4ub(mdel.vtx[v].r,mdel.vtx[v].g,mdel.vtx[v].b,mdel.vtx[v].a);

		if(!uselighting)
		{
			glColor4ub(cMap[c].r,cMap[c].g,cMap[c].b,mdel.vtx[v].a);
		}
		else
		{
			glColor4ub(cMap[c].r,cMap[c].g,cMap[c].b,255);
		}

		if(pointstyle==FRC_POINTS)
		{
			glVertex3f(mdel.vtx[v].x,mdel.vtx[v].y,mdel.vtx[v].z);
		}

		if(pointstyle==FRC_CUBES)
		{
			glPushMatrix();
			glTranslatef(mdel.vtx[v].x,mdel.vtx[v].y,mdel.vtx[v].z);
			auxSolidCube(1.0);
			glPopMatrix();
		}

		if(pointstyle==FRC_SPHERES)
		{
			glPushMatrix();
			glTranslatef(mdel.vtx[v].x,mdel.vtx[v].y,mdel.vtx[v].z);
			auxSolidSphere(0.5);
			glPopMatrix();
		}

	}

	if(zooming)
	{
		glBegin(GL_LINES);

		mdel.drawzoomcube();

		glEnd();
	}

	glEnd( );
	if(should_colorcycle)
	{
		colorcycle++;
		colorcycle%=255;
	}


	SDL_GL_SwapBuffers( );
}


//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
void setup_opengl( int width, int height )
{
	float ratio = (float) width / (float) height;

	if(uselighting)
	{
		GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 1.0 };
		GLfloat mat_shininess[] = { 50.0 };
		GLfloat light_position[] = { 1.0, 1.0, 1.0, 0.0 };

		glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
		glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
		glLightfv(GL_LIGHT0, GL_POSITION, light_position);

		glEnable(GL_LIGHTING);
		glEnable(GL_LIGHT0);
		glDepthFunc(GL_LEQUAL);
		glEnable(GL_DEPTH_TEST);

		glShadeModel( GL_SMOOTH );

		glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE );
		glEnable(GL_COLOR_MATERIAL);
	}
	else
	{
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	}

	//glCullFace( GL_BACK );
	//glFrontFace( GL_CCW );
	//glEnable( GL_CULL_FACE );

	glClearColor( 0, 0, 0, 0 );

	glViewport( 0, 0, width, height );

	glMatrixMode( GL_PROJECTION );

	glLoadIdentity( );

	//gluPerspective( 60.0, ratio, 1.0, 1024.0 );
	glFrustum(-1.0,1.0,-.75,.75,1.0,1000);

}


//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
void process_events( void )
{

	SDL_Event event;

	while( SDL_PollEvent( &event ) )
	{

		switch( event.type )
		{
			case SDL_KEYDOWN:
				handle_key_down( &event.key.keysym );
				break;

			case SDL_MOUSEMOTION:
				MouseMotionHandler(event.type, event.motion);
				break;

			case SDL_MOUSEBUTTONDOWN:
				MouseButtonHandler(event.type, event.button);
				break;

			case SDL_MOUSEBUTTONUP:
				MouseButtonHandler(event.type, event.button);
				break;

			case SDL_QUIT:
				quit_program( 0 );
				break;
		}

	}

}


//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
void handle_key_down( SDL_keysym* keysym )
{

	switch( keysym->sym )
	{
		case SDLK_ESCAPE:
			quit_program( 0 );
			break;
		case SDLK_SPACE:
			should_rotate = !should_rotate;
			break;

		case SDLK_c:
			should_colorcycle = !should_colorcycle;
			break;

		case SDLK_l:
			uselighting = !uselighting;
			break;

		case SDLK_r:
			pointstyle++;
			pointstyle%=3;
			break;

		case SDLK_p:
			pointsize++;
			break;

		case SDLK_o:
			pointsize--;
			if(pointsize==0)pointsize=1;
			break;

		case SDLK_LEFT:
			// move_x+=0.01;
			mdel.translatezoomcube(-1.0,0.0,0.0);
			break;

		case SDLK_RIGHT:
			//move_x-=0.01;
			mdel.translatezoomcube(1.0,0.0,0.0);
			break;

		case SDLK_UP:
			//move_y+=0.01;
			mdel.translatezoomcube(0.0,1.0,0.0);
			break;

		case SDLK_DOWN:
			//move_y-=0.01;
			mdel.translatezoomcube(0.0,-1.0,0.0);
			break;

		case SDLK_HOME:
			//move_z+=0.01;
			mdel.translatezoomcube(0.0,0.0,1.0);
			break;

		case SDLK_END:
			//move_z-=0.01;
			mdel.translatezoomcube(0.0,0.0,-1.0);
			break;

		case SDLK_PAGEUP:
			zooming = GL_TRUE;
			mdel.scalezoomcube(0.9,0.9,0.9);
			break;
		case SDLK_a:
			mdel.alphamode=!mdel.alphamode;
			mdel.iterate();
			break;
		case SDLK_j:
			mdel.julia=!mdel.julia;
			mdel.x1=-2.5;
			mdel.y1=-2.5;
			mdel.z1=-2.5;
			mdel.x2=2.5;
			mdel.y2=2.5;
			mdel.z2=2.5;

			mdel.iterate();
			break;

		case SDLK_PAGEDOWN:

			mdel.scalezoomcube(1.1,1.1,1.1);

			break;

		case SDLK_RETURN:
			zooming = GL_FALSE;
			mdel.zoom();

			mdel.setcube();
			mdel.setzoomcube();
			mdel.iterate();
			break;

		case SDLK_F1:
			mdel.lowcolor+=1;
			if(mdel.lowcolor==256)mdel.lowcolor=255;
			mdel.iterate();
			break;

		case SDLK_F2:
			mdel.lowcolor-=1;
			if(mdel.lowcolor==-2)mdel.lowcolor=-1;
			mdel.iterate();
			break;

		case SDLK_F3:
			mdel.highcolor+=1;
			if(mdel.highcolor==256)mdel.highcolor=255;
			mdel.iterate();
			break;

		case SDLK_F4:
			mdel.highcolor-=1;
			if(mdel.highcolor==-2)mdel.highcolor=-1;
			mdel.iterate();
			break;

		default:
			break;
	}

}


//////////////////////////////////////////////////////////////////////

void MouseButtonHandler(Uint8 type, SDL_MouseButtonEvent event)
{
	switch(event.button)
	{
		case SDL_BUTTON_LEFT:
			if (type == SDL_MOUSEBUTTONDOWN)
			{
				mouse_left = 1;
			}
			else
			{
				mouse_left = 0;
			}
			break;

		case SDL_BUTTON_RIGHT:
			if (type == SDL_MOUSEBUTTONDOWN)
			{
				mouse_right = 1;
			}
			else
			{
				mouse_right = 0;
			}
	}
}


void MouseMotionHandler(Uint8 type, SDL_MouseMotionEvent event)
{

	//xpos+=(event.xrel/50);
	//ypos+=(event.yrel/50);
}


//////////////////////////////////////////////////////////////////////
void quit_program( int code )
{

	SDL_Quit( );

	/* Exit program. */
	exit( code );
}


//////////////////////////////////////////////////////////////////////
