/*
 *   NoQuaFra - Non Quaternion 3d Fractal Generator
 *   Copyright (C) 2002 
 *
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *
 *   Billy Stiltner
 *   billystiltner@sourceforge.net
 */

/*
 * Notice! This is a stripped down version of GLAUX For the purpose of
 * drawing a solid cube and solid sphere for the NoQuaFra Program.
 * Original Licence Included Below Original GLAUX.H and GLAUX.C 
 */



/*
 * Copyright (c) 1991, 1992, 1993 Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the name of
 * Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF
 * ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL SILICON GRAPHICS BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
 * OF THIS SOFTWARE.
 */




#define SPHEREWIRE	0
#define CUBEWIRE	1
#define BOXWIRE		2
#define TORUSWIRE	3
#define CYLINDERWIRE	4
#define ICOSAWIRE	5
#define OCTAWIRE	6
#define TETRAWIRE	7
#define DODECAWIRE	8
#define CONEWIRE	9
#define SPHERESOLID	10
#define CUBESOLID	11
#define BOXSOLID	12
#define TORUSSOLID	13
#define CYLINDERSOLID	14
#define ICOSASOLID	15
#define OCTASOLID	16
#define TETRASOLID	17
#define DODECASOLID	18
#define CONESOLID	19


//PI model and modelptr taken from aux library
#define PI 3.1415926535897

/*	structure for each geometric object	*/
typedef struct model {
    GLuint list;	/*  display list to render object   */
    struct model *ptr;	/*  pointer to next object	*/
    int numParam;	/*  # of parameters		*/
    GLdouble *params;	/*  array with parameters	*/
} MODEL, *MODELPTR;

/*	array of linked lists--used to keep track of display lists 
 *	for each different type of geometric object.
 */
static MODELPTR lists[25] = {
    NULL, NULL, NULL, NULL, NULL,
    NULL, NULL, NULL, NULL, NULL,
    NULL, NULL, NULL, NULL, NULL,
    NULL, NULL, NULL, NULL, NULL,
    NULL, NULL, NULL, NULL, NULL
};

void auxSolidSphere (GLdouble radius);
void auxWireSphere (GLdouble radius);
void auxSolidCube (GLdouble size);
void auxWireCube (GLdouble size);

void drawbox(GLdouble x0, GLdouble x1, GLdouble y0, GLdouble y1,
	GLdouble z0, GLdouble z1, GLenum type);

int compareParams (GLdouble *oneArray, GLdouble *twoArray, int size);

GLuint findList (int index, GLdouble *paramArray, int size);
GLuint makeModelPtr (int index, GLdouble *sizeArray, int count);


//the following were taken from the aux library
/////////////////////////////////////////////////////

void auxWireCube (GLdouble size)
{
    GLdouble *sizeArray;
    GLuint displayList;

    sizeArray = (GLdouble *) malloc (sizeof (GLdouble) * 1);
    *sizeArray = size;
    displayList = findList (CUBEWIRE, sizeArray, 1);

    if (displayList == 0) {
	glNewList(makeModelPtr (CUBEWIRE, sizeArray, 1),
	    GL_COMPILE_AND_EXECUTE);
	    drawbox(-size/2., size/2., -size/2., size/2., 
		-size/2., size/2., GL_LINE_LOOP);
	glEndList();
    }
    else {
	glCallList(displayList);
	free (sizeArray);
    }
}

void auxSolidCube (GLdouble size)
{
    GLdouble *sizeArray;
    GLuint displayList;

    sizeArray = (GLdouble *) malloc (sizeof (GLdouble) * 1);
    *sizeArray = size;
    displayList = findList (CUBESOLID, sizeArray, 1);

    if (displayList == 0) {
	glNewList(makeModelPtr (CUBESOLID, sizeArray, 1),
	    GL_COMPILE_AND_EXECUTE);
	    drawbox(-size/2., size/2., -size/2., size/2., 
		-size/2., size/2., GL_QUADS);
	glEndList();
    }
    else {
	glCallList(displayList);
	free (sizeArray);
    }
}


void drawbox(GLdouble x0, GLdouble x1, GLdouble y0, GLdouble y1,
	GLdouble z0, GLdouble z1, GLenum type)
{
    static GLdouble n[6][3] = {
	{-1.0, 0.0, 0.0}, {0.0, 1.0, 0.0}, {1.0, 0.0, 0.0},
	{0.0, -1.0, 0.0}, {0.0, 0.0, 1.0}, {0.0, 0.0, -1.0}
    };
    static GLint faces[6][4] = {
	{ 0, 1, 2, 3 }, { 3, 2, 6, 7 }, { 7, 6, 5, 4 },
	{ 4, 5, 1, 0 }, { 5, 6, 2, 1 }, { 7, 4, 0, 3 }
    };
    GLdouble v[8][3], tmp;
    GLint i;

    if (x0 > x1) {
	tmp = x0; x0 = x1; x1 = tmp;
    }
    if (y0 > y1) {
	tmp = y0; y0 = y1; y1 = tmp; 
    }
    if (z0 > z1) {
	tmp = z0; z0 = z1; z1 = tmp; 
    }
    v[0][0] = v[1][0] = v[2][0] = v[3][0] = x0;
    v[4][0] = v[5][0] = v[6][0] = v[7][0] = x1;
    v[0][1] = v[1][1] = v[4][1] = v[5][1] = y0;
    v[2][1] = v[3][1] = v[6][1] = v[7][1] = y1;
    v[0][2] = v[3][2] = v[4][2] = v[7][2] = z0;
    v[1][2] = v[2][2] = v[5][2] = v[6][2] = z1;

    for (i = 0; i < 6; i++) {
	glBegin(type);
	glNormal3dv(&n[i][0]);
	glVertex3dv(&v[faces[i][0]][0]);
	glNormal3dv(&n[i][0]);
	glVertex3dv(&v[faces[i][1]][0]);
	glNormal3dv(&n[i][0]);
	glVertex3dv(&v[faces[i][2]][0]);
	glNormal3dv(&n[i][0]);
	glVertex3dv(&v[faces[i][3]][0]);
	glEnd();
    }
}




void auxWireSphere (GLdouble radius)
{
    GLUquadricObj *quadObj;
    GLdouble *sizeArray;
    GLuint displayList;

    sizeArray = (GLdouble *) malloc (sizeof (GLdouble) * 1);
    *sizeArray = radius;
    displayList = findList (SPHEREWIRE, sizeArray, 1);

    if (displayList == 0) {
	glNewList(makeModelPtr (SPHEREWIRE, sizeArray, 1),
	    GL_COMPILE_AND_EXECUTE);
	    quadObj = gluNewQuadric ();
	    gluQuadricDrawStyle (quadObj, GLU_LINE);
	    gluSphere (quadObj, radius, 16, 16);
	glEndList();
    }
    else {
	glCallList(displayList);
	free (sizeArray);
    }
}

void auxSolidSphere (GLdouble radius)
{
    GLUquadricObj *quadObj;
    GLdouble *sizeArray;
    GLuint displayList;

    sizeArray = (GLdouble *) malloc (sizeof (GLdouble) * 1);
    *sizeArray = radius;
    displayList = findList (SPHERESOLID, sizeArray, 1);

    if (displayList == 0) {
	glNewList(makeModelPtr (SPHERESOLID, sizeArray, 1),
	    GL_COMPILE_AND_EXECUTE);
	    quadObj = gluNewQuadric ();
	    gluQuadricDrawStyle (quadObj, GLU_FILL);
	    gluQuadricNormals (quadObj, GLU_SMOOTH);
	    gluSphere (quadObj, radius, 16, 16);
	glEndList();
    }
    else {
	glCallList(displayList);
	free (sizeArray);
    }
}

GLuint findList (int index, GLdouble *paramArray, int size) 
{
    MODELPTR endList;
    int found = 0;
    
    endList = lists[index];
    while (endList != NULL) {
	if (compareParams (endList->params, paramArray, size))
	    return (endList->list);
	endList = endList->ptr;
    }
/*  if not found, return 0 and calling routine should
 *  make a new list	
 */
    return (0);
}

int compareParams (GLdouble *oneArray, GLdouble *twoArray, int size) 
{
    int i;
    int matches = 1;

    for (i = 0; (i < size) && matches; i++) {
	if (*oneArray++ != *twoArray++)
	    matches = 0;
    }
    return (matches);
}


GLuint makeModelPtr (int index, GLdouble *sizeArray, int count)
{
    MODELPTR newModel;

    newModel = (MODELPTR) malloc (sizeof (MODEL));
    newModel->list = glGenLists (1);
    newModel->numParam = count;
    newModel->params = sizeArray;
    newModel->ptr = lists[index];
    lists[index] = newModel;

    return (newModel->list);
}